package xhttp3.internal.framed;

import java.io.IOException;
import java.util.List;
import java.util.logging.Level;
import java.util.logging.Logger;
import xhttp3.Protocol;
import xhttp3.internal.Util;
import xio.Buffer;
import xio.BufferedSink;
import xio.BufferedSource;
import xio.ByteString;
import xio.Source;
import xio.Timeout;

public final class Http2 implements Variant {
   private static final Logger logger = Logger.getLogger(FrameLogger.class.getName());
   private static final ByteString CONNECTION_PREFACE = ByteString.encodeUtf8("PRI * HTTP/2.0\r\n\r\nSM\r\n\r\n");
   static final int INITIAL_MAX_FRAME_SIZE = 16384;
   static final byte TYPE_DATA = 0;
   static final byte TYPE_HEADERS = 1;
   static final byte TYPE_PRIORITY = 2;
   static final byte TYPE_RST_STREAM = 3;
   static final byte TYPE_SETTINGS = 4;
   static final byte TYPE_PUSH_PROMISE = 5;
   static final byte TYPE_PING = 6;
   static final byte TYPE_GOAWAY = 7;
   static final byte TYPE_WINDOW_UPDATE = 8;
   static final byte TYPE_CONTINUATION = 9;
   static final byte FLAG_NONE = 0;
   static final byte FLAG_ACK = 1;
   static final byte FLAG_END_STREAM = 1;
   static final byte FLAG_END_HEADERS = 4;
   static final byte FLAG_END_PUSH_PROMISE = 4;
   static final byte FLAG_PADDED = 8;
   static final byte FLAG_PRIORITY = 32;
   static final byte FLAG_COMPRESSED = 32;

   private static IllegalArgumentException illegalArgument(String message, Object... args) {
      throw new IllegalArgumentException(Util.format(message, args));
   }

   private static IOException ioException(String message, Object... args) {
      throw new IOException(Util.format(message, args));
   }

   private static int lengthWithoutPadding(int length, byte flags, short padding) {
      if ((flags & 8) != 0) {
         --length;
      }

      if (padding <= length) {
         return (short)(length - padding);
      } else {
         Object[] flags1;
         Object[] var10000 = flags1 = new Object[2];
         flags1[0] = padding;
         var10000[1] = length;
         throw ioException("PROTOCOL_ERROR padding %s > remaining length %s", flags1);
      }
   }

   private static int readMedium(BufferedSource source) {
      return (source.readByte() & 255) << 16 | (source.readByte() & 255) << 8 | source.readByte() & 255;
   }

   private static void writeMedium(BufferedSink sink, int i) {
      sink.writeByte(i >>> 16 & 255);
      sink.writeByte(i >>> 8 & 255);
      sink.writeByte(i & 255);
   }

   public Protocol getProtocol() {
      return Protocol.HTTP_2;
   }

   public FrameReader newReader(BufferedSource source, boolean client) {
      return new Reader(source, 4096, client);
   }

   public FrameWriter newWriter(BufferedSink sink, boolean client) {
      return new Writer(sink, client);
   }

   static final class FrameLogger {
      private static final String[] TYPES = new String[]{"DATA", "HEADERS", "PRIORITY", "RST_STREAM", "SETTINGS", "PUSH_PROMISE", "PING", "GOAWAY", "WINDOW_UPDATE", "CONTINUATION"};
      private static final String[] FLAGS = new String[64];
      private static final String[] BINARY = new String[256];

      static String formatHeader(boolean inbound, int streamId, int length, byte type, byte flags) {
         String[] var5;
         String var11;
         if (type < (var5 = TYPES).length) {
            var11 = var5[type];
         } else {
            Object[] var12;
            (var12 = new Object[1])[0] = type;
            var11 = Util.format("0x%02x", var12);
         }

         boolean var10000 = inbound;
         String inbound1 = formatFlags(type, flags);
         String type1 = "%s 0x%08x %5d %-13s %s";
         Object[] flags1 = new Object[5];
         byte var6 = 0;
         String var7;
         if (var10000) {
            var7 = "<<";
         } else {
            var7 = ">>";
         }

         flags1[var6] = var7;
         flags1[1] = streamId;
         flags1[2] = length;
         flags1[3] = var11;
         flags1[4] = inbound1;
         return Util.format(type1, flags1);
      }

      static String formatFlags(byte type, byte flags) {
         if (flags == 0) {
            return "";
         } else {
            switch (type) {
               case 2:
               case 3:
               case 7:
               case 8:
                  return BINARY[flags];
               case 4:
               case 6:
                  return flags == 1 ? "ACK" : BINARY[flags];
               case 5:
               default:
                  String[] var2;
                  String var3;
                  if (flags < (var2 = FLAGS).length) {
                     var3 = var2[flags];
                  } else {
                     var3 = BINARY[flags];
                  }

                  if (type == 5 && (flags & 4) != 0) {
                     return var3.replace("HEADERS", "PUSH_PROMISE");
                  } else {
                     return type == 0 && (flags & 32) != 0 ? var3.replace("PRIORITY", "COMPRESSED") : var3;
                  }
            }
         }
      }

      static {
         int var0;
         String[] var1;
         for(var0 = 0; var0 < (var1 = BINARY).length; ++var0) {
            Object[] var11;
            (var11 = new Object[1])[0] = Integer.toBinaryString(var0);
            var1[var0] = Util.format("%8s", var11).replace(' ', '0');
         }

         String[] var10000 = FLAGS;
         var10000[0] = "";
         var10000[1] = "END_STREAM";
         byte var10 = 1;
         int[] var12;
         (var12 = new int[1])[0] = 1;
         var10000[8] = "PADDED";

         int var10001;
         StringBuilder var10002;
         for(int var2 = 0; var2 < var10; ++var2) {
            int var3 = var12[var2];
            String[] var4;
            var10000 = var4 = FLAGS;
            var10001 = var3 | 8;
            StringBuilder var5;
            var10002 = var5 = new StringBuilder;
            var5.<init>();
            var10000[var10001] = var10002.append(var4[var3]).append("|PADDED").toString();
         }

         var10000 = FLAGS;
         var10000[4] = "END_HEADERS";
         var10000[32] = "PRIORITY";
         var10000[36] = "END_HEADERS|PRIORITY";
         byte var13 = 3;
         int[] var14;
         int[] var17 = var14 = new int[3];
         var17[0] = 4;
         var17[1] = 32;
         var17[2] = 36;

         for(int var15 = 0; var15 < var13; ++var15) {
            int var16 = var14[var15];

            for(int var6 = 0; var6 < var10; ++var6) {
               int var7 = var12[var6];
               String[] var8;
               var10000 = var8 = FLAGS;
               var10001 = var7 | var16;
               int var10003 = var7 | var16;
               StringBuilder var9;
               StringBuilder var10004 = var9 = new StringBuilder;
               var9.<init>();
               var8[var10003] = var10004.append(var8[var7]).append('|').append(var8[var16]).toString();
               var10001 |= 8;
               var10002 = var9 = new StringBuilder;
               var9.<init>();
               var10000[var10001] = var10002.append(var8[var7]).append('|').append(var8[var16]).append("|PADDED").toString();
            }
         }

         for(var0 = 0; var0 < (var1 = FLAGS).length; ++var0) {
            if (var1[var0] == null) {
               var1[var0] = BINARY[var0];
            }
         }

      }
   }

   static final class ContinuationSource implements Source {
      private final BufferedSource source;
      int length;
      byte flags;
      int streamId;
      int left;
      short padding;

      public ContinuationSource(BufferedSource source) {
         this.source = source;
      }

      private void readContinuationHeader() {
         int var1 = this.streamId;
         int var2;
         this.left = var2 = Http2.readMedium(this.source);
         this.length = var2;
         byte var8 = (byte)(this.source.readByte() & 255);
         this.flags = (byte)(this.source.readByte() & 255);
         if (Http2.logger.isLoggable(Level.FINE)) {
            Logger var10000 = Http2.logger;
            int var3 = this.streamId;
            int var4 = this.length;
            byte var5 = this.flags;
            var10000.fine(Http2.FrameLogger.formatHeader(true, var3, var4, var8, var5));
         }

         int var6;
         this.streamId = var6 = this.source.readInt() & Integer.MAX_VALUE;
         if (var8 == 9) {
            if (var6 != var1) {
               throw Http2.ioException("TYPE_CONTINUATION streamId changed");
            }
         } else {
            Object[] var7;
            (var7 = new Object[1])[0] = var8;
            throw Http2.ioException("%s != TYPE_CONTINUATION", var7);
         }
      }

      public long read(Buffer sink, long byteCount) {
         int var4;
         while((var4 = this.left) == 0) {
            this.source.skip((long)this.padding);
            this.padding = 0;
            if ((this.flags & 4) != 0) {
               return -1L;
            }

            this.readContinuationHeader();
         }

         long sink1;
         if ((sink1 = this.source.read(sink, Math.min(byteCount, (long)var4))) == -1L) {
            return -1L;
         } else {
            this.left = (int)((long)this.left - sink1);
            return sink1;
         }
      }

      public Timeout timeout() {
         return this.source.timeout();
      }

      public void close() {
      }
   }

   static final class Writer implements FrameWriter {
      private final BufferedSink sink;
      private final boolean client;
      private final Buffer hpackBuffer;
      private int maxFrameSize;
      private boolean closed;
      final Hpack.Writer hpackWriter;

      Writer(BufferedSink sink, boolean client) {
         Writer var10000 = this;
         Writer var10001 = this;
         Writer var10002 = this;
         super();
         this.sink = sink;
         this.client = client;
         Buffer var3;
         var3 = new Buffer.<init>();
         var10002.hpackBuffer = var3;
         Hpack.Writer sink1;
         sink1 = new Hpack.Writer.<init>(var3);
         var10001.hpackWriter = sink1;
         var10000.maxFrameSize = 16384;
      }

      private void writeContinuationFrames(int streamId, long byteCount) {
         while(byteCount > 0L) {
            long var4;
            long var6;
            int byteCount1;
            long var10000 = var6 = byteCount - (var4 = (long)(byteCount1 = (int)Math.min((long)this.maxFrameSize, byteCount)));
            byte var3 = 9;
            byte var8;
            if (var10000 == 0L) {
               var8 = 4;
            } else {
               var8 = 0;
            }

            this.frameHeader(streamId, byteCount1, var3, var8);
            this.sink.write(this.hpackBuffer, var4);
            byteCount = var6;
         }

      }

      public synchronized void flush() {
         if (!this.closed) {
            this.sink.flush();
         } else {
            throw new IOException("closed");
         }
      }

      public synchronized void applyAndAckSettings(Settings peerSettings) {
         if (!this.closed) {
            this.maxFrameSize = peerSettings.getMaxFrameSize(this.maxFrameSize);
            if (peerSettings.getHeaderTableSize() > -1) {
               this.hpackWriter.setHeaderTableSizeSetting(peerSettings.getHeaderTableSize());
            }

            this.frameHeader(0, 0, (byte)4, (byte)1);
            this.sink.flush();
         } else {
            throw new IOException("closed");
         }
      }

      public synchronized void connectionPreface() {
         if (!this.closed) {
            if (this.client) {
               if (Http2.logger.isLoggable(Level.FINE)) {
                  Logger var10000 = Http2.logger;
                  Object[] var1;
                  (var1 = new Object[1])[0] = Http2.CONNECTION_PREFACE.hex();
                  var10000.fine(Util.format(">> CONNECTION %s", var1));
               }

               this.sink.write(Http2.CONNECTION_PREFACE.toByteArray());
               this.sink.flush();
            }
         } else {
            throw new IOException("closed");
         }
      }

      public synchronized void synStream(boolean outFinished, boolean inFinished, int streamId, int associatedStreamId, List<Header> headerBlock) {
         if (!inFinished) {
            if (!this.closed) {
               this.headers(outFinished, streamId, headerBlock);
            } else {
               throw new IOException("closed");
            }
         } else {
            throw new UnsupportedOperationException();
         }
      }

      public synchronized void synReply(boolean outFinished, int streamId, List<Header> headerBlock) {
         if (!this.closed) {
            this.headers(outFinished, streamId, headerBlock);
         } else {
            throw new IOException("closed");
         }
      }

      public synchronized void headers(int streamId, List<Header> headerBlock) {
         if (!this.closed) {
            this.headers(false, streamId, headerBlock);
         } else {
            throw new IOException("closed");
         }
      }

      public synchronized void pushPromise(int streamId, int promisedStreamId, List<Header> requestHeaders) {
         if (!this.closed) {
            this.hpackWriter.writeHeaders(requestHeaders);
            long var4;
            long var10000 = var4 = this.hpackBuffer.size();
            int requestHeaders1;
            int var10001 = requestHeaders1 = (int)Math.min((long)(this.maxFrameSize - 4), var4);
            byte var6 = 5;
            long var7;
            int var9;
            byte var10;
            long var13;
            if ((var9 = (var13 = var10000 - (var7 = (long)var10001)) == 0L ? 0 : (var13 < 0L ? -1 : 1)) == 0) {
               var10 = 4;
            } else {
               var10 = 0;
            }

            this.frameHeader(streamId, requestHeaders1 + 4, var6, var10);
            this.sink.writeInt(promisedStreamId & Integer.MAX_VALUE);
            this.sink.write(this.hpackBuffer, var7);
            if (var9 > 0) {
               long promisedStreamId1 = var4 - var7;
               this.writeContinuationFrames(streamId, promisedStreamId1);
            }

         } else {
            throw new IOException("closed");
         }
      }

      void headers(boolean outFinished, int streamId, List<Header> headerBlock) {
         if (!this.closed) {
            this.hpackWriter.writeHeaders(headerBlock);
            long var4;
            long var10000 = var4 = this.hpackBuffer.size();
            int headerBlock1;
            int var10001 = headerBlock1 = (int)Math.min((long)this.maxFrameSize, var4);
            byte var6 = 1;
            long var7;
            int var9;
            byte var10;
            long var14;
            if ((var9 = (var14 = var10000 - (var7 = (long)var10001)) == 0L ? 0 : (var14 < 0L ? -1 : 1)) == 0) {
               var10 = 4;
            } else {
               var10 = 0;
            }

            if (outFinished) {
               var10 = (byte)(var10 | 1);
            }

            this.frameHeader(streamId, headerBlock1, var6, var10);
            this.sink.write(this.hpackBuffer, var7);
            if (var9 > 0) {
               Writer var13 = this;
               long var11 = var4 - var7;
               var13.writeContinuationFrames(streamId, var11);
            }

         } else {
            throw new IOException("closed");
         }
      }

      public synchronized void rstStream(int streamId, ErrorCode errorCode) {
         if (!this.closed) {
            if (errorCode.httpCode != -1) {
               this.frameHeader(streamId, 4, (byte)3, (byte)0);
               this.sink.writeInt(errorCode.httpCode);
               this.sink.flush();
            } else {
               throw new IllegalArgumentException();
            }
         } else {
            throw new IOException("closed");
         }
      }

      public int maxDataLength() {
         return this.maxFrameSize;
      }

      public synchronized void data(boolean outFinished, int streamId, Buffer source, int byteCount) {
         if (!this.closed) {
            boolean var10000 = outFinished;
            byte outFinished1 = 0;
            if (var10000) {
               outFinished1 = (byte)1;
            }

            this.dataFrame(streamId, outFinished1, source, byteCount);
         } else {
            throw new IOException("closed");
         }
      }

      void dataFrame(int streamId, byte flags, Buffer buffer, int byteCount) {
         this.frameHeader(streamId, byteCount, (byte)0, flags);
         if (byteCount > 0) {
            this.sink.write(buffer, (long)byteCount);
         }

      }

      public synchronized void settings(Settings settings) {
         if (!this.closed) {
            this.frameHeader(0, settings.size() * 6, (byte)4, (byte)0);

            for(int var2 = 0; var2 < 10; ++var2) {
               if (settings.isSet(var2)) {
                  int var3;
                  if (var2 == 4) {
                     var3 = 3;
                  } else if (var2 == 7) {
                     var3 = 4;
                  } else {
                     var3 = var2;
                  }

                  this.sink.writeShort(var3);
                  this.sink.writeInt(settings.get(var2));
               }
            }

            this.sink.flush();
         } else {
            throw new IOException("closed");
         }
      }

      public synchronized void ping(boolean ack, int payload1, int payload2) {
         if (!this.closed) {
            Writer var10000 = this;
            Writer var10001 = this;
            Writer var10002 = this;
            Writer var10003 = this;
            byte var4 = 6;
            var10003.frameHeader(0, 8, var4, ack);
            var10002.sink.writeInt(payload1);
            var10001.sink.writeInt(payload2);
            var10000.sink.flush();
         } else {
            throw new IOException("closed");
         }
      }

      public synchronized void goAway(int lastGoodStreamId, ErrorCode errorCode, byte[] debugData) {
         if (!this.closed) {
            if (errorCode.httpCode != -1) {
               this.frameHeader(0, debugData.length + 8, (byte)7, (byte)0);
               this.sink.writeInt(lastGoodStreamId);
               this.sink.writeInt(errorCode.httpCode);
               if (debugData.length > 0) {
                  this.sink.write(debugData);
               }

               this.sink.flush();
            } else {
               throw Http2.illegalArgument("errorCode.httpCode == -1");
            }
         } else {
            throw new IOException("closed");
         }
      }

      public synchronized void windowUpdate(int streamId, long windowSizeIncrement) {
         if (!this.closed) {
            if (windowSizeIncrement != 0L && windowSizeIncrement <= 2147483647L) {
               this.frameHeader(streamId, 4, (byte)8, (byte)0);
               this.sink.writeInt((int)windowSizeIncrement);
               this.sink.flush();
            } else {
               Object[] var4;
               (var4 = new Object[1])[0] = windowSizeIncrement;
               throw Http2.illegalArgument("windowSizeIncrement == 0 || windowSizeIncrement > 0x7fffffffL: %s", var4);
            }
         } else {
            throw new IOException("closed");
         }
      }

      public synchronized void close() {
         this.closed = true;
         this.sink.close();
      }

      void frameHeader(int streamId, int length, byte type, byte flags) {
         if (Http2.logger.isLoggable(Level.FINE)) {
            Http2.logger.fine(Http2.FrameLogger.formatHeader(false, streamId, length, type, flags));
         }

         int var5;
         Object[] var6;
         if (length <= (var5 = this.maxFrameSize)) {
            if ((streamId & Integer.MIN_VALUE) == 0) {
               Http2.writeMedium(this.sink, length);
               this.sink.writeByte(type & 255);
               this.sink.writeByte(flags & 255);
               this.sink.writeInt(streamId & Integer.MAX_VALUE);
            } else {
               (var6 = new Object[1])[0] = streamId;
               throw Http2.illegalArgument("reserved bit set: %s", var6);
            }
         } else {
            Object[] var10000 = var6 = new Object[2];
            var6[0] = var5;
            var10000[1] = length;
            throw Http2.illegalArgument("FRAME_SIZE_ERROR length > %d: %d", var6);
         }
      }
   }

   static final class Reader implements FrameReader {
      private final BufferedSource source;
      private final ContinuationSource continuation;
      private final boolean client;
      final Hpack.Reader hpackReader;

      Reader(BufferedSource source, int headerTableSize, boolean client) {
         Reader var10000 = this;
         Reader var10001 = this;
         super();
         this.source = source;
         this.client = client;
         ContinuationSource var4;
         var4 = new ContinuationSource.<init>(source);
         var10001.continuation = var4;
         Hpack.Reader source1;
         source1 = new Hpack.Reader.<init>(headerTableSize, var4);
         var10000.hpackReader = source1;
      }

      private void readHeaders(FrameReader.Handler handler, int length, byte flags, int streamId) {
         if (streamId != 0) {
            boolean var5;
            if ((flags & 1) != 0) {
               var5 = true;
            } else {
               var5 = false;
            }

            short var6;
            if ((flags & 8) != 0) {
               var6 = (short)(this.source.readByte() & 255);
            } else {
               var6 = 0;
            }

            if ((flags & 32) != 0) {
               this.readPriority(handler, streamId);
               length -= 5;
            }

            FrameReader.Handler var10000 = handler;
            List var7 = this.readHeaderBlock(Http2.lengthWithoutPadding(length, flags, var6), var6, flags, streamId);
            HeadersMode handler1 = HeadersMode.HTTP_20_HEADERS;
            var10000.headers(false, var5, streamId, -1, var7, handler1);
         } else {
            throw Http2.ioException("PROTOCOL_ERROR: TYPE_HEADERS streamId == 0");
         }
      }

      private List<Header> readHeaderBlock(int length, short padding, byte flags, int streamId) {
         Reader var10000 = this;
         Reader var10001 = this;
         ContinuationSource var5;
         ContinuationSource var10002 = var5 = this.continuation;
         var5.left = length;
         var5.length = length;
         var5.padding = padding;
         var5.flags = flags;
         var10002.streamId = streamId;
         var10001.hpackReader.readHeaders();
         return var10000.hpackReader.getAndResetHeaderList();
      }

      private void readData(FrameReader.Handler handler, int length, byte flags, int streamId) {
         boolean var5;
         if ((flags & 1) != 0) {
            var5 = true;
         } else {
            var5 = false;
         }

         if ((flags & 32) == 0) {
            short var6;
            if ((flags & 8) != 0) {
               var6 = (short)(this.source.readByte() & 255);
            } else {
               var6 = 0;
            }

            Reader var10000 = this;
            Reader var10003 = this;
            int var7 = Http2.lengthWithoutPadding(length, flags, var6);
            handler.data(var5, streamId, var10003.source, var7);
            var10000.source.skip((long)var6);
         } else {
            throw Http2.ioException("PROTOCOL_ERROR: FLAG_COMPRESSED without SETTINGS_COMPRESS_DATA");
         }
      }

      private void readPriority(FrameReader.Handler handler, int length, byte flags, int streamId) {
         if (length == 5) {
            if (streamId != 0) {
               this.readPriority(handler, streamId);
            } else {
               throw Http2.ioException("TYPE_PRIORITY streamId == 0");
            }
         } else {
            Object[] var5;
            (var5 = new Object[1])[0] = length;
            throw Http2.ioException("TYPE_PRIORITY length: %d != 5", var5);
         }
      }

      private void readPriority(FrameReader.Handler handler, int streamId) {
         int var3;
         boolean var4;
         if (((var3 = this.source.readInt()) & Integer.MIN_VALUE) != 0) {
            var4 = true;
         } else {
            var4 = false;
         }

         Reader var10002 = this;
         int var5 = var3 & Integer.MAX_VALUE;
         handler.priority(streamId, var5, (var10002.source.readByte() & 255) + 1, var4);
      }

      private void readRstStream(FrameReader.Handler handler, int length, byte flags, int streamId) {
         if (length == 4) {
            if (streamId != 0) {
               int var6;
               ErrorCode length1;
               if ((length1 = ErrorCode.fromHttp2(var6 = this.source.readInt())) != null) {
                  handler.rstStream(streamId, length1);
               } else {
                  Object[] handler1;
                  (handler1 = new Object[1])[0] = var6;
                  throw Http2.ioException("TYPE_RST_STREAM unexpected error code: %d", handler1);
               }
            } else {
               throw Http2.ioException("TYPE_RST_STREAM streamId == 0");
            }
         } else {
            Object[] var5;
            (var5 = new Object[1])[0] = length;
            throw Http2.ioException("TYPE_RST_STREAM length: %d != 4", var5);
         }
      }

      private void readSettings(FrameReader.Handler handler, int length, byte flags, int streamId) {
         if (streamId != 0) {
            throw Http2.ioException("TYPE_SETTINGS streamId != 0");
         } else if ((flags & 1) != 0) {
            if (length == 0) {
               handler.ackSettings();
            } else {
               throw Http2.ioException("FRAME_SIZE_ERROR ack frame should be empty!");
            }
         } else {
            Object[] var7;
            if (length % 6 != 0) {
               (var7 = new Object[1])[0] = length;
               throw Http2.ioException("TYPE_SETTINGS length %% 6 != 0: %s", var7);
            } else {
               Settings flags1;
               flags1 = new Settings.<init>();

               for(streamId = 0; streamId < length; streamId += 6) {
                  short var5;
                  short var10000 = var5 = this.source.readShort();
                  int var6 = this.source.readInt();
                  switch (var10000) {
                     case 2:
                        if (var6 != 0 && var6 != 1) {
                           throw Http2.ioException("PROTOCOL_ERROR SETTINGS_ENABLE_PUSH != 0 or 1");
                        }
                        break;
                     case 3:
                        var5 = 4;
                        break;
                     case 4:
                        var5 = 7;
                        if (var6 < 0) {
                           throw Http2.ioException("PROTOCOL_ERROR SETTINGS_INITIAL_WINDOW_SIZE > 2^31 - 1");
                        }
                        break;
                     case 5:
                        if (var6 < 16384 || var6 > 16777215) {
                           (var7 = new Object[1])[0] = var6;
                           throw Http2.ioException("PROTOCOL_ERROR SETTINGS_MAX_FRAME_SIZE: %s", var7);
                        }
                  }

                  flags1.set(var5, 0, var6);
               }

               handler.settings(false, flags1);
            }
         }
      }

      private void readPushPromise(FrameReader.Handler handler, int length, byte flags, int streamId) {
         if (streamId != 0) {
            short var5;
            if ((flags & 8) != 0) {
               var5 = (short)(this.source.readByte() & 255);
            } else {
               var5 = 0;
            }

            Reader var10002 = this;
            int var6 = this.source.readInt() & Integer.MAX_VALUE;
            handler.pushPromise(streamId, var6, var10002.readHeaderBlock(Http2.lengthWithoutPadding(length + -4, flags, var5), var5, flags, streamId));
         } else {
            throw Http2.ioException("PROTOCOL_ERROR: TYPE_PUSH_PROMISE streamId == 0");
         }
      }

      private void readPing(FrameReader.Handler handler, int length, byte flags, int streamId) {
         if (length == 8) {
            if (streamId == 0) {
               Reader var10001 = this;
               int var6 = this.source.readInt();
               length = var10001.source.readInt();
               boolean flags1;
               if ((flags & 1) != 0) {
                  flags1 = true;
               } else {
                  flags1 = false;
               }

               handler.ping(flags1, var6, length);
            } else {
               throw Http2.ioException("TYPE_PING streamId != 0");
            }
         } else {
            Object[] var5;
            (var5 = new Object[1])[0] = length;
            throw Http2.ioException("TYPE_PING length != 8: %s", var5);
         }
      }

      private void readGoAway(FrameReader.Handler handler, int length, byte flags, int streamId) {
         Object[] var6;
         if (length >= 8) {
            if (streamId == 0) {
               flags = this.source.readInt();
               int var10000 = streamId = this.source.readInt();
               length -= 8;
               ErrorCode var5;
               if ((var5 = ErrorCode.fromHttp2(var10000)) != null) {
                  ByteString streamId1 = ByteString.EMPTY;
                  if (length > 0) {
                     streamId1 = this.source.readByteString((long)length);
                  }

                  handler.goAway(flags, var5, streamId1);
               } else {
                  (var6 = new Object[1])[0] = streamId;
                  throw Http2.ioException("TYPE_GOAWAY unexpected error code: %d", var6);
               }
            } else {
               throw Http2.ioException("TYPE_GOAWAY streamId != 0");
            }
         } else {
            (var6 = new Object[1])[0] = length;
            throw Http2.ioException("TYPE_GOAWAY length < 8: %s", var6);
         }
      }

      private void readWindowUpdate(FrameReader.Handler handler, int length, byte flags, int streamId) {
         Object[] var5;
         if (length == 4) {
            long length1;
            if ((length1 = (long)this.source.readInt() & 2147483647L) != 0L) {
               handler.windowUpdate(streamId, length1);
            } else {
               (var5 = new Object[1])[0] = length1;
               throw Http2.ioException("windowSizeIncrement was 0", var5);
            }
         } else {
            (var5 = new Object[1])[0] = length;
            throw Http2.ioException("TYPE_WINDOW_UPDATE length !=4: %s", var5);
         }
      }

      public void readConnectionPreface() {
         if (!this.client) {
            ByteString var2 = this.source.readByteString((long)Http2.CONNECTION_PREFACE.size());
            Object[] var1;
            if (Http2.logger.isLoggable(Level.FINE)) {
               Logger var10000 = Http2.logger;
               (var1 = new Object[1])[0] = var2.hex();
               var10000.fine(Util.format("<< CONNECTION %s", var1));
            }

            if (!Http2.CONNECTION_PREFACE.equals(var2)) {
               (var1 = new Object[1])[0] = var2.utf8();
               throw Http2.ioException("Expected a connection header but was %s", var1);
            }
         }
      }

      public boolean nextFrame(FrameReader.Handler handler) {
         Reader var10000;
         try {
            var10000 = this;
            this.source.require(9L);
         } catch (IOException var6) {
            return false;
         }

         int var2;
         if ((var2 = Http2.readMedium(var10000.source)) >= 0 && var2 <= 16384) {
            byte var3 = (byte)(this.source.readByte() & 255);
            byte var4 = (byte)(this.source.readByte() & 255);
            int var5 = this.source.readInt() & Integer.MAX_VALUE;
            if (Http2.logger.isLoggable(Level.FINE)) {
               Http2.logger.fine(Http2.FrameLogger.formatHeader(true, var5, var2, var3, var4));
            }

            switch (var3) {
               case 0:
                  this.readData(handler, var2, var4, var5);
                  break;
               case 1:
                  this.readHeaders(handler, var2, var4, var5);
                  break;
               case 2:
                  this.readPriority(handler, var2, var4, var5);
                  break;
               case 3:
                  this.readRstStream(handler, var2, var4, var5);
                  break;
               case 4:
                  this.readSettings(handler, var2, var4, var5);
                  break;
               case 5:
                  this.readPushPromise(handler, var2, var4, var5);
                  break;
               case 6:
                  this.readPing(handler, var2, var4, var5);
                  break;
               case 7:
                  this.readGoAway(handler, var2, var4, var5);
                  break;
               case 8:
                  this.readWindowUpdate(handler, var2, var4, var5);
                  break;
               default:
                  this.source.skip((long)var2);
            }

            return true;
         } else {
            Object[] var7;
            (var7 = new Object[1])[0] = var2;
            throw Http2.ioException("FRAME_SIZE_ERROR: %s", var7);
         }
      }

      public void close() {
         this.source.close();
      }
   }
}
